@@ -4,6 +4,7 @@ import logging
 from urllib.parse import urljoin
 from base64 import urlsafe_b64decode
 import requests
+from flask import Request, abort
 
 GCP_PROJECT = os.environ["GCP_PROJECT"]
 FUNCTION_REGION = os.environ["FUNCTION_REGION"]
@@ -22,11 +23,12 @@ def get_auth_header(audience):
     if os.getenv("LOCAL"):
         from subprocess import Popen, PIPE
 
-        return (
+        token = (
             Popen(["gcloud", "auth", "print-identity-token"], stdout=PIPE)
             .communicate()[0][:-1]
             .decode()
         )
+        return {"Authorization": "Bearer " + token}
     token_url = (
         "http://metadata.google.internal"
         "/computeMetadata/v1/instance/service-accounts/default/identity"
@@ -38,13 +40,17 @@ def get_auth_header(audience):
     return {"Authorization": "Bearer " + token}
 
 
-def call(name, data=None):
+def call(name, **kwargs):
     domain = f"https://{FUNCTION_REGION}-{GCP_PROJECT}.cloudfunctions.net"
     url = urljoin(domain, name)
-    logging.info("CALL: %s %s", name, data)
-    r = requests.post(url, headers=get_auth_header(url), json=data)
+    logging.info("CALL: %s", name)
+    logging.info(kwargs)
+    r = requests.post(url, headers=get_auth_header(url), json=kwargs)
+    resp = r.json()
+    logging.info("RESPONSE: %s", name)
+    logging.info(resp)
     r.raise_for_status()
-    return r.json()
+    return resp
 
 
 def user_info(request):
@@ -56,5 +62,18 @@ def user_info(request):
     return None
 
 
-def response_message(message):
-    return {"message": message}
+def authorize(permission, asset):
+    def wrap(func):
+        def wrapped_func(request: Request):
+            uid = user_info(request)["user_id"]
+            granted = call(
+                "check_access",
+                uid=uid,
+                asset=asset(**request.json),
+                permission=permission
+            )["granted"]
+            if not granted:
+                abort(403)
+            func(request)
+        return wrapped_func
+    return wrap
